@@ -3,7 +3,7 @@
 
 # DUE_VERSION_COMPATIBILITY_TRACKING=2.0.0
 
-# Copyright 2020 NVIDIA Corporation. All rights reserved.
+# Copyright 2020,2021 NVIDIA Corporation. All rights reserved.
 #
 #  SPDX-License-Identifier:     MIT
 
@@ -28,17 +28,20 @@ function fxnHelp()
     echo "Usage  : $(basename "$0"): --name <reponame> [OPTIONS]"
     echo ""
     echo " OPTIONS:"
-    echo "   --name <reponame>          Specify repo name to use."
+    echo "   --name <reponame>          Specify repo name to use. New repos should have unique names."
     echo "   --create-repo              Create empty local repository."
-    echo "     --architecture <arch>    Architecture type for repository. Default: [ $USE_ARCHITECTURE]"
+    echo "     --architecture <arch>    Architecture type for repository. Default: [ $USE_ARCHITECTURE ]"
     echo "   --delete-repo              Remove all traces of --name <repo>"
+    echo "   --update-repo              Rebuild index of current repo contents."
     echo ""
     echo "   --add <deb or dir of debs> Add deb file or directory holding debs to repository."
+    echo "                               For multiple single debs, repeat --add."
+    echo "                                Ex: --add foo.deb --add bar.deb"
     echo "   --delete-package <term>    Remove <term> from repo."
     echo "                               If '<term>' wildcards can be passed. See --help-examples."
     echo ""
-    echo "   --disable-repo             Remove apt files referencing repo, keep repo dir."
-    echo "   --enable-repo              Add apt files to reference existing repo dir."
+    echo "   --disable-repo             Remove apt files referencing <reponame>, but keep the repository."
+    echo "   --enable-repo              Set apt files to reference <reponame>."
     echo "   --no-apt-update            Skip default behavior of apt-updating on exit."
     echo ""
     echo " Information:"
@@ -53,22 +56,28 @@ function fxnHelp()
 # Examples of use
 function fxnHelpExamples()
 {
+    local userIs="$(whoami)"
+
     echo "Examples:"
     echo ""
     echo " Create repository foo:"
-    echo "   $0 --name /home/$(whoami)/foo --create-repo"
+    echo "   $0 --name /home/${userIs}/foo --create-repo"
     echo ""
-    echo " Add a package or directory of packages:"
-    echo "   $0 --name /home/$(whoami)/foo -add foo.deb"
-    echo "   $0 --name /home/$(whoami)/foo -add ./mydebs/"
+    echo " Create repository foo using packages in ~/pkgdir... "
+    echo "   ...and make it active with  /etc/apt/sources.list.d/${LOCAL_REPO_SOURCES_LIST}:"
+    echo "   $0 --name /home/${userIs}/foo --create-repo --add /home/${userIs}/pkgdir --enable-repo"
+    echo ""
+    echo " Add a package(s) or directory of packages:"
+    echo "   $0 --name /home/${userIs}/foo -add foo.deb --add bar.deb"
+    echo "   $0 --name /home/${userIs}/foo -add ./mydebs/"
     echo ""
     echo " Delete package bar.deb from repository:"
-    echo "   $0 --name /home/$(whoami)/foo --delete-package bar.deb"
+    echo "   $0 --name /home/${userIs}/foo --delete-package bar.deb"
     echo " Delete all packages starting with baz (note the single quotes!):"
-    echo " $0 --name /home/$(whoami)/foo --delete-package 'baz*'"
+    echo "   $0 --name /home/${userIs}/foo --delete-package 'baz*'"
     echo ""
     echo " Delete repository foo:"
-    echo "   $0 --name /home/$(whoami)/foo --delete-repo"
+    echo "   $0 --name /home/${userIs}/foo --delete-repo"
     echo""
 
 }
@@ -117,28 +126,33 @@ function fxnAddRepositoryPackage()
 {
     local toAdd="$1"
 
-    if [ ! -e "$toAdd" ];then
-        fxnERR "Failed to find [ $toAdd ] to add to local repository. Exiting."
-        exit 1
-    fi
+    for debFile in $toAdd ; do
+        if [ ! -e "$debFile" ];then
+            fxnERR "Failed to find [ $debFile ] to add to local repository. Exiting."
+            exit 1
+        fi
 
-    if [ -d "$1" ];then
-        fxnPP "Copying ${toAdd}/*.deb to $REPO_POOL_DIR"
-        fxnEC cp -f "${toAdd}/"*.deb "$REPO_POOL_DIR" || exit 1
-    elif [ -f "$1" ];then
-        fxnPP "Copying [ $toAdd ] to $REPO_POOL_DIR"
-        fxnEC cp -f "$toAdd" "$REPO_POOL_DIR" || exit 1
-
-    else
-        fxnERR "Could not find [ $1 ] to add to local package repository."
-        exit 1
-    fi
+        if [ -d "$debFile" ];then
+            fxnPP "Copying ${debFile}/*.deb to $REPO_POOL_DIR"
+            fxnEC cp -f "${debFile}/"*.deb "$REPO_POOL_DIR" || exit 1
+        elif [ -f "$debFile" ];then
+            fxnPP "Copying [ $debFile ] to $REPO_POOL_DIR"
+            fxnEC cp -f "$debFile" "$REPO_POOL_DIR" || exit 1
 
+        else
+            fxnERR "Could not find [ $debFile ] to add to local package repository."
+            exit 1
+        fi
+    done
     # refresh Package files
     fxnUpdateRepository
 
     fxnListRepoPool
 
+    # With packages in, apt updates are safe.
+    DO_APT_UPDATE="TRUE"
+
+
 }
 
 # Takes: name of package to delete
@@ -189,7 +203,7 @@ function fxnUpdateRepository()
 
     cd "$LOCAL_PACKAGE_REPOSITORY_ROOT" || exit 1
 
-    fxnPP "Updating local repository at $LOCAL_PACKAGE_REPOSITORY_PATH"
+    fxnPP "Updating local repository at [ $LOCAL_PACKAGE_REPOSITORY_PATH ]"
     #   fxnEC apt-ftparchive --arch $USE_ARCHITECTURE  packages pool |
     fxnEC apt-ftparchive packages pool | \
         tee "${LOCAL_REPOSITORY_BIN_DIR}"/Packages | \
@@ -200,7 +214,7 @@ function fxnUpdateRepository()
 -o APT::FTPArchive::Release::Label=localPackageRepository \
 -o APT::FTPArchive::Release::Suite=local-repo \
 -o APT::FTPArchive::Release::Version=1.0 \
--o APT::FTPArchive::Release::Codename=local-due-repo \
+-o APT::FTPArchive::Release::Codename=$RELEASE_NAME \
 -o APT::FTPArchive::Release::Architectures=$USE_ARCHITECTURE \
 -o APT::FTPArchive::Release::Description=LocalPackageRepository \
 -o APT::FTPArchive::Release::Components=main"
@@ -227,8 +241,32 @@ function fxnManageAPTConfig()
 {
 
     local localRepoSourcesList=""
+    #
+    # Sanity check that a valid reposiory was passed.
+    #
+    if [  -e "$LOCAL_PACKAGE_REPOSITORY_ROOT" ];then
+        if [  ! -e "$REPO_DEBS_DIR" ] || \
+               [ ! -e "$REPO_POOL_DIR" ] || \
+               [ ! -e "$LOCAL_REPOSITORY_BIN_DIR" ];then
+            echo -e "\nFailed to find one of:\n [ $REPO_DEBS_DIR ]\n [ $REPO_POOL_DIR ]\n [ $LOCAL_REPOSITORY_BIN_DIR ]\n Try deleting and re-creating the repository? \n"
+            fxnERR "Invalid repository [ $LOCAL_PACKAGE_REPOSITORY_ROOT ]"
+            exit 1
+        fi
+    else
+        fxnERR "Failed to find local repository at [ $LOCAL_PACKAGE_REPOSITORY_ROOT ] "
+		if [ "$1" = "ENABLE" ];then
+			# Cannot activate what is not there
+			exit 1
+		fi
+		# Disable, however, is still possible if this is cleaning up a broken configuration.
+    fi
 
     if [ "$1" = "ENABLE" ];then
+        #
+        # Always start clean. Could be a new enable for a new repo.
+        #
+        fxnManageAPTConfig "DISABLE" > /dev/null
+
         # Update the sources.list file
         localRepoSourcesList="${LOCAL_PACKAGE_REPOSITORY_ROOT}/${LOCAL_REPO_SOURCES_LIST}"
         if [ ! -e /etc/apt/sources.list.d/"${LOCAL_REPO_SOURCES_LIST}" ];then
@@ -255,7 +293,7 @@ function fxnManageAPTConfig()
             cat <<EOF > /tmp/apt-pref
 # Created by $(whoami) using $0 building from $(pwd) on $(date)
 Package: *
-Pin: release n=local-due-repo
+Pin: release n=$RELEASE_NAME
 Pin-Priority: 990  >
 EOF
             fxnEC sudo mv /tmp/apt-pref "$LOCAL_APT_PREFERENCES_FILE" || exit 1
@@ -272,7 +310,8 @@ EOF
         else
             fxnPP "Skipping APT update as no packages have been added yet, and 'Failed to stat' errors will be seen."
             fxnPP "Use $0 --name $LOCAL_REPO_NAME --add <deb or directory of debs> to add packages."
-            # Exit at this point, having set up the sources.list and apt preferences files to
+            DO_APT_UPDATE="FALSE"
+            # At this point the sources.list and apt preferences files are set up to
             # reference and prioritize the local repository.
         fi
 
@@ -305,7 +344,7 @@ function fxnSetPaths()
 {
     local repoName="$1"
 
-    if [ "$2" != "SKIP_CHECK" ];then
+    if [ "$2" != "SKIP_REPO_EXISTS_CHECK" ];then
         # Repo creation code calls this, so do not exit before
         # it has had a chance to create the directory.
         if [ ! -e "$repoName" ];then
@@ -362,7 +401,7 @@ function fxnCreateLocalPackageRepository()
 
     # Set paths and disable checking for the lack of a repository,
     # as we are about to address that...
-    fxnSetPaths "$LOCAL_REPO_NAME" "SKIP_CHECK"
+    fxnSetPaths "$LOCAL_REPO_NAME" "SKIP_REPO_EXISTS_CHECK"
 
     if [ ! -e "$LOCAL_PACKAGE_REPOSITORY_ROOT" ];then
         fxnPP "Creating directories for local Debian package repository at: $LOCAL_PACKAGE_REPOSITORY_PATH"
@@ -389,6 +428,8 @@ function fxnCreateLocalPackageRepository()
 #                                                #
 ##################################################
 
+# unless specified otherwise, always update apt.
+DO_APT_UPDATE="TRUE"
 
 if [ "$#" = "0" ];then
     # Require an argument for action.
@@ -397,15 +438,6 @@ if [ "$#" = "0" ];then
     exit 0
 fi
 
-# Default to local architecture
-USE_ARCHITECTURE=$( dpkg-architecture --query DEB_TARGET_ARCH )
-
-LOCAL_REPOSITORY_DISTRIBUTION="local-due-repo"
-LOCAL_REPOSITORY_COMPONENT="main"
-LOCAL_REPO_SOURCES_LIST="local-build-repo.list"
-LOCAL_APT_PREFERENCES_FILE="/etc/apt/preferences.d/10local-build-repo"
-# unless specified otherwise, always update apt.
-DO_APT_UPDATE="TRUE"
 #
 # Gather arguments and set action flags for processing after
 # all parsing is done. The only functions that should get called
@@ -424,6 +456,8 @@ do
         --name )
             # specify name of repository to reference
             LOCAL_REPO_NAME="$2"
+			# Create a name for the release from the name of the repository.
+			RELEASE_NAME=$( basename $LOCAL_REPO_NAME )			
             shift
             ;;
 
@@ -447,11 +481,12 @@ do
             ;;
 
         --add )
-            ADD_TO_REPO="$2"
+            # add a package to a list of packages to add
             if [ ! -e "$2" ];then
                 fxnERR "Failed to find [ $ADD_TO_REPO ] to add to local repository. Exiting."
                 exit 1
             fi
+            ADD_TO_REPO+=" $2 "
             shift
             ;;
 
@@ -487,9 +522,9 @@ do
             exit 0
             ;;
 
-        --repo-update )
-            fxnUpdateRepository
-            exit
+        --update-repo )
+            # Rebuild package files and update the repository. Put Apt files under /etc
+            DO_UPDATE_REPOSITORY="TRUE"
             ;;
 
         -h|--help)
@@ -515,6 +550,18 @@ do
 
 done
 
+#
+# Set repository values based on any user input
+#
+
+
+# Default to local architecture (set here as help uses it )
+USE_ARCHITECTURE=$( dpkg-architecture --query DEB_TARGET_ARCH )
+LOCAL_REPOSITORY_DISTRIBUTION="${RELEASE_NAME}"
+LOCAL_REPOSITORY_COMPONENT="main"
+LOCAL_REPO_SOURCES_LIST="due-local-${RELEASE_NAME}-repo.list"
+LOCAL_APT_PREFERENCES_FILE="/etc/apt/preferences.d/10due-local-${RELEASE_NAME}-repo"
+
 #
 # As all other commands require the existence of a repository,
 # the first option is to create one.
@@ -524,6 +571,27 @@ if [ "$CREATE_LOCAL_REPO" = "TRUE" ];then
 fi
 
 
+if [ "$DELETE_LOCAL_REPO" = "TRUE" ];then
+	# Delete takes a slightly different set paths
+	# Generate repo file names. Do not check for validity as
+	# the repo may be gone, but apt config files still exist.
+	fxnSetPaths "$LOCAL_REPO_NAME" "SKIP_REPO_EXISTS_CHECK"
+	
+    fxnPP "Removing local package repository [ $LOCAL_REPO_NAME ] and associated files."
+    if [ -e "$LOCAL_REPO_NAME" ];then
+        fxnPP "Deleting [ $LOCAL_REPO_NAME ]"
+        rm -rf "$LOCAL_REPO_NAME"
+    else
+        # Print as message, not error as this has been handled.
+        fxnPP "Done. No repository directory at  [ $LOCAL_REPO_NAME ]."
+    fi
+    # Clean up apt preferences/sources.list in case the repo
+    # directory was manually deleted but these weren't.
+    fxnManageAPTConfig "DISABLE"
+
+	exit
+fi
+
 #
 # Set paths to various locations and sanity check for this repository.
 # Has to be after fxnCreateLocalRepository so that the script actually has
@@ -531,6 +599,10 @@ fi
 #
 fxnSetPaths "$LOCAL_REPO_NAME"
 
+if [ "$DO_UPDATE_REPOSITORY" = "TRUE" ];then
+    fxnUpdateRepository
+fi
+
 if [ "$ENABLE_APT_CONFIG" = "TRUE" ];then
     # Add local sources.list references
     fxnManageAPTConfig "ENABLE"
@@ -554,19 +626,6 @@ if [ "$DO_LIST_REPO" = "TRUE" ];then
     fxnListRepoPool
 fi
 
-if [ "$DELETE_LOCAL_REPO" = "TRUE" ];then
-    fxnPP "Removing local package repository [ $LOCAL_REPO_NAME ] and associated files."
-    if [ -e "$LOCAL_REPO_NAME" ];then
-        fxnPP "Deleting [ $LOCAL_REPO_NAME ]"
-        rm -rf "$LOCAL_REPO_NAME"
-    else
-        # Print as message, not error as this has been handled.
-        fxnPP "Done. No repository directory at  [ $LOCAL_REPO_NAME ]."
-    fi
-    # Clean up apt preferences/sources.list in case the repo
-    # directory was manually deleted but these weren't.
-    fxnManageAPTConfig "DISABLE"
-fi
 
 if [ "$DO_APT_UPDATE" = "TRUE" ];then
     fxnPP "Updating APT."
